@@ -24,7 +24,7 @@ static int decode_ContentType_ChangeCipherSpec(ssl,dir,seg,data)
     else{
       ssl->r_state=SSL_ST_SENT_CHANGE_CIPHER_SPEC;
     }
-    
+
     printf("\n");
     return(0);
 
@@ -36,7 +36,6 @@ static int decode_ContentType_Alert(ssl,dir,seg,data)
   Data *data;
   {
 
- 
    int r;
 
    if(ssl->record_encryption==REC_CIPHERTEXT){
@@ -63,7 +62,6 @@ static int decode_ContentType_Alert(ssl,dir,seg,data)
 	 printf("\n");
    }
    return(0);
-	
 
   }
 static int decode_ContentType_Handshake(ssl,dir,seg,data)
@@ -78,7 +76,7 @@ static int decode_ContentType_Handshake(ssl,dir,seg,data)
     int r;
     UINT4 t,l;
     int rs=0;
-    Data d;	
+    Data d;
 
     if(ssl->record_encryption==REC_CIPHERTEXT){
       printf("\n");
@@ -88,17 +86,17 @@ static int decode_ContentType_Handshake(ssl,dir,seg,data)
     while(data->len>0){
       SSL_DECODE_UINT8(ssl,0,0,data,&t);
       SSL_DECODE_UINT24(ssl,0,0,data,&l);
-	
+
       if(data->len<l){
         fprintf(stderr,"Error: short handshake length: expected %d got %d\n",
         l,data->len);
         ERETURN(R_EOD);
       }
-	
+
       d.data=data->data;
       d.len=l;
       data->len-=l;
-      data->data+=l;	
+      data->data+=l;
       P_(P_HL){
 	if(!rs){
 	  printf("\n");
@@ -117,18 +115,18 @@ static int decode_ContentType_application_data(ssl,dir,seg,data)
   Data *data;
   {
 
-  
+
     int r;
     Data d;
-    		
+
     SSL_DECODE_OPAQUE_ARRAY(ssl,"data",data->len,0,data,&d);
 
-    P_(P_AD){	
+    P_(P_AD){
 	    print_data(ssl,&d);
     }
     else {
 	printf("\n");
-    }      
+    }
     return(0);
 
   }
@@ -183,11 +181,11 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
     extern decoder cipher_suite_decoder[];
     extern decoder compression_method_decoder[];
     extern decoder extension_decoder[];
-	
+
     printf("\n");
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,0,0,data,&vj);
-    SSL_DECODE_UINT8(ssl,0,0,data,&vn);    
+    SSL_DECODE_UINT8(ssl,0,0,data,&vn);
 
     P_(P_HL) {explain(ssl,"Version %d.%d ",vj,vn);
         printf("\n");
@@ -204,10 +202,13 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
         exdump(ssl,"resume ",&session_id);
     }
 
+    ssl_process_client_session_id(ssl,ssl->decoder,session_id.data,
+      session_id.len);
+
     P_(P_HL){
 	SSL_DECODE_UINT16(ssl,"cipher Suites len",0,data,&cslen);
         explain(ssl,"cipher suites\n");
-    
+
         odd = cslen % 2;
         if(odd) {
             printf("Wrong cipher suites length, fixing ...\n");
@@ -239,7 +240,7 @@ static int decode_HandshakeType_ClientHello(ssl,dir,seg,data)
     	if (ssl_decode_switch(ssl,extension_decoder,ex,dir,seg,data) == R_NOT_FOUND) {
 	  decode_extension(ssl,dir,seg,data);
     	  P_(P_RH){
-    	    explain(ssl, "Extension type: %s not yet implemented in ssldump", ex);
+     	    explain(ssl, "Extension type: %u not yet implemented in ssldump\n", ex);
     	  }
     	  continue;
     	}
@@ -257,7 +258,7 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
   {
 
     int r;
-    Data rnd,session_id;	
+    Data rnd,session_id;
     UINT4 vj,vn,exlen,ex;
 
     extern decoder extension_decoder[];
@@ -265,14 +266,14 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
     printf("\n");
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,0,0,data,&vj);
-    SSL_DECODE_UINT8(ssl,0,0,data,&vn);    
+    SSL_DECODE_UINT8(ssl,0,0,data,&vn);
 
-    ssl->version=vj*256+vn;	        
+    ssl->version=vj*256+vn;
     P_(P_HL) {explain(ssl,"Version %d.%d ",vj,vn);
         printf("\n");
    }
-    	 
-	       
+
+
     SSL_DECODE_OPAQUE_ARRAY(ssl,"random",32,P_ND,data,&rnd);
     ssl_set_server_random(ssl->decoder,rnd.data,rnd.len);
     SSL_DECODE_OPAQUE_ARRAY(ssl,"session_id",-32,P_HL,data,&session_id);
@@ -280,7 +281,7 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
       0,data,&ssl->cipher_suite);
     P_(P_HL){
      explain(ssl,"cipherSuite ");
-     ssl_print_cipher_suite(ssl,ssl->version,P_HL,ssl->cipher_suite);      
+     ssl_print_cipher_suite(ssl,ssl->version,P_HL,ssl->cipher_suite);
     }
     ssl_find_cipher(ssl->cipher_suite,&ssl->cs);
 
@@ -299,7 +300,7 @@ static int decode_HandshakeType_ServerHello(ssl,dir,seg,data)
     	if (ssl_decode_switch(ssl,extension_decoder,ex,dir,seg,data) == R_NOT_FOUND) {
 	  decode_extension(ssl,dir,seg,data);
     	  P_(P_RH){
-    	    explain(ssl, "Extension type: %s not yet implemented in ssldump", ex);
+    	    explain(ssl, "Extension type: %u not yet implemented in ssldump,\n", ex);
     	  }
     	  continue;
     	}
@@ -321,7 +322,7 @@ static int decode_HandshakeType_Certificate(ssl,dir,seg,data)
     UINT4 len;
     Data cert;
     int r;
-  
+
     printf("\n");
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT24(ssl,"certificates len",0,data,&len);
@@ -346,7 +347,7 @@ static int decode_HandshakeType_ServerKeyExchange(ssl,dir,seg,data)
 
    int r;
 
-    printf("\n");			      
+    printf("\n");
     ssl_update_handshake_messages(ssl,data);
    if(ssl->cs){
      P_(P_ND){
@@ -357,8 +358,8 @@ static int decode_HandshakeType_ServerKeyExchange(ssl,dir,seg,data)
      switch(ssl->cs->kex){
 	case KEX_DH:
 	  SSL_DECODE_OPAQUE_ARRAY(ssl,"DH_p",-((1<<15)-1),P_ND,data,0);
-	  SSL_DECODE_OPAQUE_ARRAY(ssl,"DH_g",-((1<<15)-1),P_ND,data,0);	  
-	  SSL_DECODE_OPAQUE_ARRAY(ssl,"DH_Ys",-((1<<15)-1),P_ND,data,0);	  
+	  SSL_DECODE_OPAQUE_ARRAY(ssl,"DH_g",-((1<<15)-1),P_ND,data,0);
+	  SSL_DECODE_OPAQUE_ARRAY(ssl,"DH_Ys",-((1<<15)-1),P_ND,data,0);
 	  break;
 	case KEX_RSA:
 	  SSL_DECODE_OPAQUE_ARRAY(ssl,"RSA_modulus",-((1<<15)-1),P_ND,data,0);
@@ -368,7 +369,7 @@ static int decode_HandshakeType_ServerKeyExchange(ssl,dir,seg,data)
       INDENT_POP;
       SSL_DECODE_OPAQUE_ARRAY(ssl,"signature",-((1<<15)-1),P_ND,data,0);
    }
-     
+
    return(0);
 
   }
@@ -383,7 +384,7 @@ static int decode_HandshakeType_CertificateRequest(ssl,dir,seg,data)
     UINT4 len;
     Data ca;
     int r;
-    
+
     printf("\n");
     ssl_update_handshake_messages(ssl,data);
     SSL_DECODE_UINT8(ssl,"certificate_types len",0,data,&len);
@@ -398,7 +399,7 @@ static int decode_HandshakeType_CertificateRequest(ssl,dir,seg,data)
     SSL_DECODE_UINT16(ssl,"certificate_authorities len",0,data,&len);
     while(len){
       SSL_DECODE_OPAQUE_ARRAY(ssl,"certificate_authorities",
-        -((1<<15)-1),0,data,&ca); 
+        -((1<<15)-1),0,data,&ca);
       explain(ssl,"certificate_authority\n");
       INDENT_INCR;
       sslx_print_dn(ssl,&ca,P_HL);
@@ -446,7 +447,7 @@ static int decode_HandshakeType_ClientKeyExchange(ssl,dir,seg,data)
 
    int r;
    Data pms;
-	
+
     printf("\n");
     ssl_update_handshake_messages(ssl,data);
    if(ssl->cs){
@@ -456,10 +457,10 @@ static int decode_HandshakeType_ClientKeyExchange(ssl,dir,seg,data)
 	   if(ssl->version > 768) {
 	           SSL_DECODE_OPAQUE_ARRAY(ssl,"EncryptedPreMasterSecret",-(1<<15-1),
 	             P_ND,data,&pms);
-		     
+
 	        }
 	        else {
-	           SSL_DECODE_OPAQUE_ARRAY(ssl,"EncryptedPreMasterSecret",data->len,P_ND,data,&pms);   
+	           SSL_DECODE_OPAQUE_ARRAY(ssl,"EncryptedPreMasterSecret",data->len,P_ND,data,&pms);
 	        }
 	        ssl_process_client_key_exchange(ssl,
 	        ssl->decoder,pms.data,pms.len);
@@ -485,7 +486,7 @@ static int decode_HandshakeType_Finished(ssl,dir,seg,data)
 
    int r;
 
-    printf("\n");   
+    printf("\n");
    switch(ssl->version){
      case 0x300:
        SSL_DECODE_OPAQUE_ARRAY(ssl,"md5_hash",16,P_ND,data,0);
@@ -2537,7 +2538,7 @@ static int decode_extension_server_name(ssl,dir,seg,data)
 	if (ssl_decode_switch(ssl,server_name_type_decoder,t,dir,seg,data) == R_NOT_FOUND) {
 	  decode_server_name(ssl,dir,seg,data);
 	  P_(P_RH){
-	    explain(ssl, "Server Name type: %s not yet implemented in ssldump", t);
+	    explain(ssl, "Server Name type: %u not yet implemented in ssldump\n", t);
 	  }
 	  continue;
 	}
@@ -2654,6 +2655,11 @@ decoder extension_decoder[] = {
 		"next_protocol_negotiation",
 		decode_extension
 	},
+	{
+		0xff01,
+		"renegotiation_info",
+		decode_extension
+	},
 
 {-1}
 };
@@ -2669,6 +2675,17 @@ static int decode_server_name_type_host_name(ssl,dir,seg,data)
     printf(": %.*s",l,data->data);
 
     /* Possibly use data->data to set/modify ssl->server_name */
+	if (l!=0)
+	{
+		char* server_name;
+		server_name = calloc(l+1,sizeof(char));
+		if (server_name != NULL)
+		{
+			if (ssl->server_name) free(ssl->server_name);
+			memcpy(server_name,data->data,l);
+			ssl->server_name = server_name;
+		}
+	}
 
     data->len-=l;
     data->data+=l;
